/*
GUIBEGIN
WINDOW , 73, 172, 261, 166, POPUP|CAPTION|SYSMENU|MINBOX|MAXBOX|THICK, , Pick ODBC Database software
	FONT 8, 400, MS Shell Dlg
	PUSH 189, 150, 32, 14, TABSTOP, , Ok, , OK
	PUSH 226, 150, 32, 14, TABSTOP, , Cancel, , Cancel
	LIST 0, 0, 258, 153, NOTIFY|BORDER|VSCROLL|HSCROLL|TABSTOP, , Datasource
	TEXT 2, 152, 183, 11, SUNKEN|GROUP, , Description
DEND
GUIEND
*/

/* An OPTIONS "NOSOURCE" may appear here, but _not_ before the GUIBEGIN/GUIEND.
 * Any other options you wish to set (and all other initialization), other than
 * LABELCHECK should be set in the Create() function below.
 */
OPTIONS "NOSOURCE"

/* ******************* ODBC_PickSource.rex *********************
 * This is a REXX GUI child (object) script that presents a (modal)
 * window containing a listbox that we fill with the names of all ODBC
 * Data Sources on this system. The user will pick his choice from
 * the list, and then click on the Ok button. We then call GuiWake()
 * and pass the name of the chosen Data Source as the "GuiSignal".
 * If the user instead clicks the Cancel button, or closes the window,
 * then we DROP GuiSignal. If there are no data sources at all, we
 * set GuiSignal to an empty string. If there's an ODBC error, the
 * ERROR condition is raised, and for other errors (such as REXX GUI)
 * then SYNTAX is raised. (NOTE: We assume that caller is trapping
 * SYNTAX for REXX GUI errors, and ERROR for ODBC errors, and has
 * initialized the ODBC add-on functions).
 *
 * Before calling this script, the ODBC Environment handle must be
 * gotten. When the caller does a CreateObject(), we expect him to pass
 * 2 extra args; the ODBC Environment handle, and the default Data
 * source name. NOTE: If the default data source is already installed
 * on this system, then we don't bother presenting any window to the
 * user. We simply call GuiWake() and set GuiSignal to that default name.
 */





/* Called when our creator calls CreateObject() to create an instance
 * (object) of this Window Layout script. (ie, The main script is doing
 * a CreateObject call where the first arg is the name of this Window
 * Layout script).
 *
 * NOTE: This is a child script that has been modified slightly. We
 * expect the creator to create us as so:
 *
 * CreateObject("Choose_Database.rex", , , DefaultSource)
 *
 * DefaultSource =	The default Data Source name.
 */
Create:
	USE ARG DefaultSource

	/* Raise SYNTAX for any error to a GUI call we make in this script. */
	GuiErr = "SYNTAX"
	GuiHeading = 1

	/* Raise ERROR for any error to a ODBC call we make in this script. */
	OdbcErr = "ERROR"
	OdbcHeading = 1

	DO
		/* Get the names/descriptions of the installed data sources. */
		OdbcGetDSNList("dsn.", "description.") 
		IF dsn.0 == 0 THEN DO
			GuiSay("No database drivers available", 'STOP')

			/* Have the main script return from GuiGetMsg with no choice. */
			GuiWake("")
		
			RETURN
		END

		CATCH ERROR
			GuiWake("")
			RETURN
	END

	/* Check if the default data source is one of those listed. */
	temp = TRANSLATE(STRIP(DefaultSource))
	DO i = 1 TO dsn.0
		IF temp == TRANSLATE(STRIP(dsn.i)) THEN LEAVE
	END

	/* Was it listed? */
	IF i <= dsn.0 THEN DO

		/* It is listed, so there's no need to open a dialog to let
		 * the user choose a source. Just pass the default source
		 * to GuiWake().
		 */
		GuiWake(DefaultSource)
		RETURN
	END

	/* We have to present the dialog of data sources. NOTE: We pass -1 to
	 * create it modal. We also omit the second arg because we don't want the
	 * window displayed before we fill in the "Datasource" listbox.
	 */
	GuiCreateWindow(, -1)

	/* NOTE: Reginald has set the variable "GuiWindow" to our window
	 * handle if everything went well.
 	 */

 	/* Fill in the Datasource listbox with all the data source names. */
	DO i = 1 TO Dsn.0
		GuiSendMsg("Datasource", "ADDSTRING", , Dsn.i)
	END

	/* Now show the window. */
	GuiSetCtlPlacement(GuiWindow,,,,,,'NORMAL')

	/* Return to our creator and let him do the (modal) message loop. */
	RETURN




/* Called by reginald after the user clicks on the close box. The window
 * will be gone when we RETURN.
 */
DestroyWindow:

/* Called when our creator DROPs us. */
Destroy:
	/* Let's make sure that our window is destroyed. We do that by calling
	 * GuiDestroyWindow. NOTE: It is safe to call GuiDestroyWindow even if our
	 * GuiCreateWindow never succeeded. If we haven't called GuiWake() to set
	 * GuiSignal, then REXX GUI automatically sets GuiObject to our object name,
	 * and DROPs GuiSignal.
	 */
	GuiDestroyWindow()
	RETURN





/* Called by Reginald when our OK button is clicked. */
WM_CLICK_Ok:
	/* Get the user's choice in the Datasource listbox. */
	GuiGetCtlValue("Datasource")

	/* If a choice was made, then call GuiWake, using that name as the GuiSignal. */
	IF EXISTS("Datasource") THEN GuiWake(Datasource)

	RETURN





/* Called by Reginald when our Cancel button is clicked. */
WM_CLICK_Cancel:
	/* Mimic the user clicking on the close box. */
	GuiSendMsg(, "POST CLOSE")

	RETURN





/* Called by Reginald when an item in our Datasource listbox is clicked. */
WM_SELECT_Datasource:
	/* Get which item # he selected in the Datasource listbox. */
	GuiSendMsg("Datasource", "GETCURSEL")

	/* Display that Description string. */
	GuiAddCtlText("Description", Description.GuiSignal)

	RETURN