/* This script shows how to use the RxZip add-on DLL to create a ZIP archive
 * from the contents of some REXX Variables.
 */

OPTIONS "C_CALL LABELCHECK"

/* Register the Rxzip add-on DLL */
LIBRARY rxzip

/* Tell RxZip to raise SYNTAX condition for any failure with a Zip function */
ZipErr = "SYNTAX"
ZipHeading = 1

/* Here are some variables which we'll compress into our zip file */
Data1 =	"This is some data." || "0D0A0D0A"X || "This is line two."|| "0D0A"X
Data2 =	"This is a test of the RxZip add-on dll." || "0D0A0D0A"X || "Hopefully this worked!"|| "0D0A"X

DO
	/* Create a ZIP archive on disk named "MyTest.zip". Store the ZIP handle
	 * in a variable named "ZipHandle"
	 */
	ZipCreate("ZipHandle", "MyTest.zip")

	/* Add the contents of the Data1 to the ZIP, and give it the name "data1.txt". */
	ZipAdd(ZipHandle,  Data1, "MEMORY", "data1.txt")

	/* Add the contents of the Data2 to the ZIP, and give it the name "data2.txt". */
	ZipAdd(ZipHandle,  Data2, "MEMORY", "data2.txt")

	/* Here we could call ZipAdd to add disk files, the contents of more variables,
	 * or the output of a pipe, to the ZIP archive.
	 */

	/* We're done adding files to our ZIP in memory, so just let the FINALLY section close it. */

	CATCH SYNTAX
		CONDITION('M')

	FINALLY
		/* Make sure any ZIP archive we created above is closed. */
		IF EXISTS("ZIPHANDLE") THEN ZipClose(ZipHandle)
END
